%% HOW TO WRANGLE FLOW CYTOMETRY DATA
%
% Jue Wang
% Updated 20140813
% Created 20130506
% 
% This tutorial demonstrates the functions in the Flow Cytometry Toolkit, a
% package that Bo Hua and I put together to facilitate analysis of large
% amounts of flow cytometry data.
% 
% The functions are designed to be modular so you can pick and choose what
% is most useful for a given experiment. That said, there are a few basic
% code patterns for using the toolkit, so read/run this file to learn them.
% 
% This tutorial has a few sections:
% 
%   LOADING DATA
%   PLOTTING DATA
%   MANIPULATING DATA
%   SEGMENTATION
% 

%% LOADING DATA
%% LOADING DATA FROM SINGLE FCS FILE

% This is the folder where we put the data
datadir = 'data/';

% This loads the data into a convenient struct-based format.
% This data format is assumed by all the functions in the toolkit.
[data,meta] = fcsparse([datadir 'd1p1_Well 012_300.fcs'])

% This loads the data while renaming 'FITC_530_30_LogH' to 'yfp'
% It also renames channels for 'mch', 'cfp', and 'bfp'.
[data,meta] = fcsparse([datadir 'd1p1_Well 012_300.fcs'],'rename')

%% ADVANCED DATA LOADING

% This loads the data from well A4, from the plate named 'd1p1' The plate
% name argument should be the string before the first underscore in the fcs
% filenames.
[data,meta] = fcsreadwell(datadir,'d1p1','a4');

% Other equivalent ways of typing this are:
% [data,meta] = fcsreadwell(datadir,'d1p1','A04');
% [data,meta] = fcsreadwell(datadir,'d1p1',coord2well(1,4));
% 
% The well represented by each fcs file is inferred from the header (as
% opposed to the filename, which may vary in format). Since reading headers
% can be slow, the function caches a lookup table in DATADIR the first time
% it is run.
% 
% If you run this on a partial set of data and add files later, you
% should delete 'wellfilenameindex.mat' so it can regenerate the cache.

%% PLOTTING DATA
%% PLOT 1-D HISTOGRAM

X = linspace(0,4,100);

% compute histogram
Y = histc(log10(data.yfp),X);

figure;
plot(X,Y);

%% Compute smoothed histogram
X = linspace(0,4,100);

% compute histogram
Y = ksdensity(log10(data.yfp),X);

figure;
plot(X,Y);
%%
close all

%% PLOT 2-D HISTOGRAM HEATMAP

figure;
fcsdensity(log10(data.mch),log10(data.yfp));

%% change number of bins
figure
fcsdensity(log10(data.mch),log10(data.yfp),'nbins',100);

%% contour plot
figure
fcsdensity(log10(data.mch),log10(data.yfp),'plotstyle','contour');

%% scatter plot
figure
fcsdensity(log10(data.mch),log10(data.yfp),'plotstyle','scatter');

%% scatter plot, changing various options
figure
fcsdensity(log10(data.mch),log10(data.yfp),'plotstyle','scatter',...
    'plotoptions',{'.','markersize',3});

%%
close all

%% MANIPULATING DATA
% The following functions are used to perform basic operations on our
% struct data type.
% 
%   FCSCLEAN - Removes NaNs, Infs, and 0's from a data structure.
% 
%   FCSAPPEND - Appends the data for each fluorescence channel of one data
%   struct to another.
% 
%   FCSNUMEL - Returns the number of events in a data struct.
% 
%   FCSSELECT - Returns a new data struct that is a subset of the original,
%   based on a vector of indices or a logical array. The elements chosen
%   are exactly what you'd get if you used MATLAB matrix indexing.
% 
%   FCSSTRUCT - Constructs an empty data struct. For use with FCSAPPEND to
%   pool data.
% 
%   FCSTHIN - Returns a randomly chosen subset of the data. Uses to plot
%   data that has too many events to see accurately.
% 
% For full usage details, check their code.

%% SEGMENTING FLOW CYTOMETRY DATA
% To distinguish 2 labeled populations of cells, FCTOOLKIT allows you to
% use 3 methods:
% 
% 1. THRESHOLDING - This is easiest, and more than adequate if you have
% clean data. Functions you can use:
% 
%   FCS2DTHRESH(data, channels, thresholds) applies 2 thresholds on a
%   scatterplot and returns 4 populations in the following order: x-ON /
%   y-OFF; y-ON / x-OFF; ON-ON; OFF-OFF. In flow cytometry jargon, you
%   could call these singlet 1, singlet 2, doublets, and dark cells.
% 
% 2. GATING - this is manually drawing polygons on scatterplots to segment
% events.
% 
%   UIGETGATE(data, channels) makes a scatterplot of your data and lets you
%   define as many polygons as you want, and saves the results for
%   filtering purposes
% 
%   APPLYGATE(data, gatestruct) filters data according to polygons you
%   defined using UIGETGATE
% 
% 3. AUTOMATIC SEGMENTATION. We fit a 2-gaussian mixture model to the 2D
% histogram of a fluorescent marker versus forward scatter. If there are 2
% colors, this procedure is done separately on both colors and the results
% are intersected to define the 4 populations (singlet 1; singlet 2;
% doublets; dark cells).
% 
%   FCSSEGMENT(data, channels, ...) does a 2-color gaussian mixture
%   segmentation. An example of usage is given below.

%% THRESHOLDING
% 1-d thresholding

[data,meta] = fcsparse([datadir 'd1p1_Well 012_300.fcs'],'rename')

% eyeball a good threshold by looking at a scatterhist plot
figure
scatterhist(log10(data.mch),log10(data.yfp))

%% apply threshold
mcherryon = fcsselect(data,log10(data.mch)>=1.6);
mcherryoff = fcsselect(data,log10(data.mch)<1.6);

figure
plot(log10(mcherryon.mch),log10(mcherryon.yfp),'.',...
    log10(mcherryoff.mch),log10(mcherryoff.yfp),'.')

%% 2D thresholding
% this returns a struct array containing data for 4 subpopulations: x-ON /
%   y-OFF; y-ON / x-OFF; ON-ON; OFF-OFF.
% Right now it assumes that the thresholds are log10 of the data. In the
% future I will make this optional.
subpops = fcs2dthresh(data,{'mch' 'yfp'},[1.6 2]);

figure
plotsubpops(subpops,{'mch' 'yfp'})

%% GATING (MANUAL SEGMENTATION) WITH UIGETGATE AND APPLYGATE FUNCTIONS

[data,meta] = fcsparse([datadir 'd1p1_Well 012_300.fcs'],'rename')

% Thin the data for display
thindata = fcsthin(data,50000)

% Define gates on mCherry versus YFP. When the figure appears, click the
% vertices of a few polygons, pressing [ENTER] after each polygon. Press
% [ENTER] again after you finish drawing to tell the function you're done.
figure

% Use log-log axes
gatestruct = uigetgate(fcsthin(data,100000),{'mch','yfp'},'log');   
xlabel('log10(mCherry)')
ylabel('log10(YFP)')

% The FCSTHIN function isn't necessary--it just thins the scatterplot so it
% isn't a solid mess of dots. For example you could also call
% 
% gatestruct = uigetgate(data,{'mch','yfp'},'log');   
% 

%% UIGETGATE returns a struct
% this struct contains all the information needed to filter out a subset of
% the data, including the coordinates of the polygon you drew, the names of
% the channels, and whether it was on linear or log-transformed axes
gatestruct

%% use APPLYGATE and a gate struct to filter the data
newdata = applygate(data, gatestruct)

% notice that newdata is a 1x2 struct array. the first (or n) element(s)
% contains the events you gated, and the last element always contains the
% remaining, ungated, events.

% make a plot of the gated data
figure
plotsubpops(newdata,{'mch','yfp'})

%%
close all        
        
%% AUTOMATIC SEGMENTATION

[data,meta] = fcsparse([datadir 'd1p1_Well 012_300.fcs'],'rename')

% This will use a mixed-gaussian fitting algorithm to automatically segment
% your data into singlet,singlet,doublet,off populations.
subpops = fcssegment(data,{'mch','yfp'});

% This plots the subpopulations contained in SUBPOPS
figure
plotsubpops(subpops,{'mch','yfp'})

